#!/bin/bash

usage()
{
   echo "USAGE: `basename ${0}` <product_dir>"
}


get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

product_dir=${1}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the product directory"
   usage
   exit 1
fi


# we don't know anything about ups flavor yet
OS=`uname`
VRS=`uname -r`
MYDIR=${OS}-${VRS}
upspkgver=v5_0_2
updpkgver=v4_7_5

get_this_dir

DB=${product_dir}
echo "ups database is ${DB}"

blddir=${product_dir}/ups/${upspkgver}/${MYDIR}

mkdir ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

cd ${blddir}
pwd
tar xf ${product_dir}/ups/${upspkgver}/tar/ups502.tar.bz2
if [ ! -e Makefile ]
then
   echo "ERROR: cannot find ${blddir}/Makefile"
   exit 1
fi

export UPS_DIR=${blddir}
export PRODUCTS=${DB}
if [ -z ${UPS_SHELL} ]
then
   export UPS_SHELL=${SHELL}
   echo "setting UPS_SHELL to ${SHELL}"
fi
make all
echo "make all has finished"

cd ${product_dir}/ups/${upspkgver}

# declare 
if [ ${OS} = "Darwin" ]
then
  flvr=`${UPS_DIR}/bin/ups flavor -2`
else
  flvr=`${UPS_DIR}/bin/ups flavor`
fi
upspkgdir=${product_dir}/ups/${upspkgver}/${flvr}
mv ${blddir} ${upspkgdir}
export UPS_DIR=${upspkgdir}
echo "now declaring ups ${upspkgver} in ${DB}"
if [ ${OS} = "Darwin" ]
then
  ${UPS_DIR}/bin/ups declare ups ${upspkgver} -r ups/${upspkgver}/${flvr} -2 -m ups.table -C -z ${DB}
else
  ${UPS_DIR}/bin/ups declare ups ${upspkgver} -r ups/${upspkgver}/${flvr} -4 -m ups.table -C -z ${DB}
fi
${UPS_DIR}/bin/ups declare ups ${upspkgver} -C -c

# also declare upd
echo "check to see if we should declare upd"
source ${DB}/setup
updver=`ups list -Kversion upd -z ${DB}`
if [ -z ${updver} ]
then
   upddir=${DB}/upd/${updpkgver}/NULL
   if [ -d ${upddir} ]
   then
      ${UPS_DIR}/bin/ups declare -c upd ${updpkgver} -r upd/${updpkgver}/NULL -0 -m upd.table
   else
      echo "ERROR: cannot find ${upddir}"
      exit 1
   fi
else
   echo "upd has already been declared"
fi

exit 0
